#include "stdafx.h"
#include "resource.h"
#include <string>
#include <iostream>
#include <fstream>
#include <CommCtrl.h>
#include <sstream>
#include <stdlib.h>

HINSTANCE hInst;
BOOL advancedMenuOpen;
DEVMODE dm = {0};

std::string gameExe, gameDir, gameIcon, gameTitle;
std::string defGameDir, iniFile, cmdLine, gameFile;
std::string gameFileExt, cvarName, cvarValue, currentLine;
std::string resolutionString;
std::ofstream fileStream;
int prevPelsWidth = 0, prevPelsHeight = 0;

BOOL RunProgram (LPSTR lpApplicationName, LPSTR lpCommandLine, BOOL waitForApplication)
{
	STARTUPINFO info={sizeof(info)};
	PROCESS_INFORMATION processInfo;
	if (CreateProcess(lpApplicationName, lpCommandLine, NULL, NULL, TRUE, 0, NULL, NULL, &info, &processInfo))
	{
		if (waitForApplication)
			::WaitForSingleObject(processInfo.hProcess, INFINITE);
		CloseHandle(processInfo.hProcess);
		CloseHandle(processInfo.hThread);
		return TRUE;
	}
	return FALSE;
}

void CenterWindow (HWND hwndDlg)
{
	RECT Rect;
	GetWindowRect(hwndDlg, &Rect);
	int cx = Rect.right - Rect.left;
	int cy = Rect.bottom - Rect.top;
	int windowPositionX = GetSystemMetrics(SM_CXSCREEN) / 2 - cx / 2;
	int windowPositionY = GetSystemMetrics(SM_CYSCREEN) / 2 - cy / 2;
	SetWindowPos(hwndDlg, HWND_TOPMOST, windowPositionX, windowPositionY, cx, cy, SWP_SHOWWINDOW);
}

std::string IntToStr (int val)
{
	std::string result = static_cast<std::ostringstream*>( &(std::ostringstream() << val) )->str();
	return result;
}

std::string GetComboboxText (HWND hDlg, int nIDDlgItem)
{
	char tmpMsg[128];
	std::string comboboxText;
	int nIndex = SendMessage(GetDlgItem(hDlg, nIDDlgItem), CB_GETCURSEL, 0, 0);
	SendMessage(GetDlgItem(hDlg, nIDDlgItem), CB_GETLBTEXT, nIndex, (LPARAM)tmpMsg);
	comboboxText = tmpMsg;
	return comboboxText;
}

std::string ReadIni (LPCSTR lpAppName, LPCSTR lpKeyName, LPCSTR lpDefault, LPCSTR lpFileName)
{
	char tmpStr[128];
	std::string iniValue;
	GetPrivateProfileString(lpAppName, lpKeyName, lpDefault, tmpStr, 128, lpFileName);
	iniValue = tmpStr;
	return iniValue;
}

std::string GetCurrentDir ()
{
	char tmpStr[_MAX_PATH];
	std::string currentDir;
	GetCurrentDirectory(_MAX_PATH, tmpStr);
	currentDir = tmpStr;
	return currentDir + "/";
}

void RemoveWhitespace (std::string& str)
{
	for (size_t i = 0; i < str.length(); i++)
	{
		if (str[i] == ' ' || str[i] == '\n' || str[i] == '\t')
		{
			str.erase(i, 1);
			i--;
		}
	}
}

BOOL FileExists (const char* fileName)
{
    std::ifstream infile(fileName);
    return infile.good();
}

BOOL PopulateGameDirList (HWND hwndDlg)
{
	WIN32_FIND_DATA ffd;
	HANDLE hFind = INVALID_HANDLE_VALUE;
	hFind = FindFirstFile((GetCurrentDir() + "\\*").c_str(), &ffd);
	if (INVALID_HANDLE_VALUE == hFind) 
	{
		return FALSE;
	}
	do
	{
		if (ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (_stricmp(ffd.cFileName, ".") && _stricmp(ffd.cFileName, "..") && _stricmp(ffd.cFileName, "sys") && _stricmp(ffd.cFileName, "qindiegl") && _stricmp(ffd.cFileName, "autoload")
				&& _stricmp(ffd.cFileName, "data") && _stricmp(ffd.cFileName, "edit"))
				SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_GAMEDIR), CB_ADDSTRING, 0, (LPARAM) _T(ffd.cFileName));
		}
	}
	while (FindNextFile(hFind, &ffd) != 0);
	return TRUE;
}

BOOL AddCustomGameData ()
{
	WIN32_FIND_DATA ffd;
	HANDLE hFind = INVALID_HANDLE_VALUE;
	hFind = FindFirstFile((gameDir + "\\*").c_str(), &ffd);
	if (INVALID_HANDLE_VALUE != hFind) 
	{
		do
		{
			if (!(ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				gameFile = ffd.cFileName;
				gameFileExt = gameFile.substr(gameFile.find_last_of(".") + 1);
				if (!_stricmp(gameFileExt.c_str(), "zipfs") || !_stricmp(gameFileExt.c_str(), "zip") || !_stricmp(gameFileExt.c_str(), "pk3"))
				{
					fileStream << ("mount -o ro,p5 \"$GAME\"/" + gameFile + " /" + "\n");
					fileStream << ("mount -o rw,p5 \"$GAME\"/" + gameFile + " /" + "\n");
				}
			}
		}
		while (FindNextFile(hFind, &ffd) != 0);
	}
	hFind = INVALID_HANDLE_VALUE;
	hFind = FindFirstFile((gameDir + "\\base\\*").c_str(), &ffd);
	if (INVALID_HANDLE_VALUE != hFind) 
	{
		do
		{
			if (!(ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				gameFile = ffd.cFileName;
				gameFileExt = gameFile.substr(gameFile.find_last_of(".") + 1);
				if (!_stricmp(gameFileExt.c_str(), "zipfs") || !_stricmp(gameFileExt.c_str(), "zip") || !_stricmp(gameFileExt.c_str(), "pk3"))
				{
					fileStream << ("mount -o ro,p5 \"$GAME\"/base/" + gameFile + " /" + "\n");
					fileStream << ("mount -o rw,p5 \"$GAME\"/base/" + gameFile + " /" + "\n");
				}
			}
		}
		while (FindNextFile(hFind, &ffd) != 0);
	}
	return TRUE;
}

BOOL AddAutoloadGameData ()
{
	WIN32_FIND_DATA ffd;
	HANDLE hFind = INVALID_HANDLE_VALUE;
	hFind = FindFirstFile("autoload\\*", &ffd);
	if (INVALID_HANDLE_VALUE != hFind) 
	{
		do
		{
			if (!(ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				gameFile = ffd.cFileName;
				gameFileExt = gameFile.substr(gameFile.find_last_of(".") + 1);
				if (!_stricmp(gameFileExt.c_str(), "zipfs") || !_stricmp(gameFileExt.c_str(), "zip") || !_stricmp(gameFileExt.c_str(), "pk3"))
				{
					fileStream << ("mount -o ro,p7 autoload/" + gameFile + " /" + "\n");
					fileStream << ("mount -o rw,p7 autoload/" + gameFile + " /" + "\n");
				}
			}
		}
		while (FindNextFile(hFind, &ffd) != 0);
	}
	hFind = INVALID_HANDLE_VALUE;
	hFind = FindFirstFile("autoload\\base\\*", &ffd);
	if (INVALID_HANDLE_VALUE != hFind) 
	{
		do
		{
			if (!(ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				gameFile = ffd.cFileName;
				gameFileExt = gameFile.substr(gameFile.find_last_of(".") + 1);
				if (!_stricmp(gameFileExt.c_str(), "zipfs") || !_stricmp(gameFileExt.c_str(), "zip") || !_stricmp(gameFileExt.c_str(), "pk3"))
				{
					fileStream << ("mount -o ro,p7 autoload/base/" + gameFile + " /" + "\n");
					fileStream << ("mount -o rw,p7 autoload/base/" + gameFile + " /" + "\n");
				}
			}
		}
		while (FindNextFile(hFind, &ffd) != 0);
	}
	return TRUE;
}

void SaveLauncherSettings (HWND hwndDlg)
{
	// Save Launcher Settings
	WritePrivateProfileString("Launcher", "GameDir", GetComboboxText(hwndDlg, COMBOBOX_MAIN_GAMEDIR).c_str(), "./launcher.ini");
	WritePrivateProfileString("Launcher", "Renderer", GetComboboxText(hwndDlg, COMBOBOX_MAIN_RENDERER).c_str(), "./launcher.ini");
	WritePrivateProfileString("Launcher", "Resolution", GetComboboxText(hwndDlg, COMBOBOX_MAIN_RESOLUTION).c_str(), "./launcher.ini");
	if (SendMessage(GetDlgItem(hwndDlg, RADIO_MAIN_16BIT), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Launcher", "ColorDepth", "16", "./launcher.ini");
	else
		WritePrivateProfileString("Launcher", "ColorDepth", "32", "./launcher.ini");
	
	if (SendMessage(GetDlgItem(hwndDlg, RADIO_MAIN_FULLSCREEN), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Launcher", "Fullscreen", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Launcher", "Fullscreen", "0", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_MAIN_AUTOLOAD), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Launcher", "UseAutoload", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Launcher", "UseAutoload", "0", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_MAIN_ALWAYSSHOW), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Launcher", "SkipDialog", "0", "./launcher.ini");
	else
		WritePrivateProfileString("Launcher", "SkipDialog", "1", "./launcher.ini");
}

void SaveAdvancedLauncherSettings (HWND hwndDlg)
{
	// Save Advanced Launcher Settings
	WritePrivateProfileString("Advanced", "FullscreenRefreshRate", GetComboboxText(hwndDlg, COMBOBOX_ADV_FSRR).c_str(), "./launcher.ini");
	if (SendMessage(GetDlgItem(hwndDlg, RADIO_ADV_HIGH), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "TextureDetail", "High", "./launcher.ini");
	else if (SendMessage(GetDlgItem(hwndDlg, RADIO_ADV_MED), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "TextureDetail", "Medium", "./launcher.ini");
	else
		WritePrivateProfileString("Advanced", "TextureDetail", "Low", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOPIXELFOG), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "DisablePixelFog", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Advanced", "DisablePixelFog", "0", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWARETNL), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "DisableHardwareTnL", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Advanced", "DisableHardwareTnL", "0", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWARE3DSND), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "DisableHardware3DSound", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Advanced", "DisableHardware3DSound", "0", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWARETEXTRANSFORMS), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "DisableHardwareTextureTransforms", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Advanced", "DisableHardwareTextureTransforms", "0", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWAREENVMAPPING), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "DisableHardwareEnvironmentMapping", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Advanced", "DisableHardwareEnvironmentMapping", "0", "./launcher.ini");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOCOMPILEDVERTARRAYS), BM_GETCHECK, 0, 0))
		WritePrivateProfileString("Advanced", "DisableCompiledVertexArrays", "1", "./launcher.ini");
	else
		WritePrivateProfileString("Advanced", "DisableCompiledVertexArrays", "0", "./launcher.ini");
}

void PlayGame (HWND hwndDlg)
{
	// Prepare Launch Commands
	defGameDir = "duke";
	gameExe = "./prism3d.exe";
	gameDir = GetComboboxText(hwndDlg, COMBOBOX_MAIN_GAMEDIR).c_str();
	iniFile = gameDir + "/GameInfo.ini";
	gameIcon = ReadIni("GameInfo", "Icon", "duke/sys/duke.ico", iniFile.c_str());
	gameTitle = ReadIni("GameInfo", "Title", "Duke Nukem - Manhattan Project", iniFile.c_str());
	cmdLine = gameExe + " -game " + gameDir + " -icon " + gameIcon + " -title \"" + gameTitle + "\"";

	// DirectX 9 Renderer
	if (!_stricmp(GetComboboxText(hwndDlg, COMBOBOX_MAIN_RENDERER).c_str(), "DirectX 9"))
	{
		DeleteFile("./sys/gl.dll");
		CopyFile("./qindiegl/gl.dll", "./sys/gl.dll", false);
		CopyFile("./qindiegl/opengl32.nolog.dll", "./opengl32.dll", false);
	}
	else
	{
		DeleteFile("./sys/gl.dll");
		CopyFile("./qindiegl/gl_original.dll", "./sys/gl.dll", false);
		DeleteFile("./opengl32.dll");
	}

	// Generate Sys.ini
	if (!FileExists((gameDir + "/sys/").c_str()))
		CreateDirectory((gameDir + "/sys/").c_str(), NULL);
	if (!FileExists((gameDir + "/sys/sys.ini").c_str()))
	{
		fileStream.open((gameDir + "/sys/sys.ini").c_str());
		fileStream << ("# Duke Nukem game dll list\n");
		fileStream << ("# Order is important here!\n\n");
		fileStream << ("sys/skmod\n");
		fileStream << ("duke/sys/duke_base\n\n");
		fileStream.close();
	}

	// Generate Init.cfg
	if (FileExists((gameDir + "/init.cfg").c_str()))
		DeleteFile((gameDir + "/init.cfg").c_str());
	fileStream.open((gameDir + "/init.cfg").c_str());
	fileStream << ("# initialize the virtual file system and start the game\n\n");
	fileStream << ("# Mount core engine data\n");
	fileStream << ("mount -o ro,p0 " + defGameDir + "/base/db_cor.zipfs /" + "\n\n");
	fileStream << ("# Mount base game data\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e01.zipfs /" + "\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e02.zipfs /" + "\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e03.zipfs /" + "\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e04.zipfs /" + "\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e05.zipfs /" + "\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e06.zipfs /" + "\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e07.zipfs /" + "\n");
	fileStream << ("mount -o ro,p1 " + defGameDir + "/base/db_e08.zipfs /" + "\n\n");
	fileStream << ("# Mount language specific override data (if present)\n");
	fileStream << ("mount -o ro,p2 " + defGameDir + "/base/db_lang.zipfs /" + "\n\n");
	fileStream << ("# Mount patch data\n");
	fileStream << ("mount -o ro,p3 " + defGameDir + "/base/db_p101.zipfs /" + "\n\n");
	fileStream << ("# Mount base game folder\n");
	fileStream << ("mount -o rw,p4 " + defGameDir + "/base/ /" + "\n");
	fileStream << ("mount -o rw,fb " + defGameDir + "/base/ /" + "\n\n");
	fileStream << ("# Mount custom game data\n");

	AddCustomGameData();

	fileStream << ("\n# Mount custom game folder\n");
	fileStream << ("mount -o rw,p6 \"$GAME\"/ /\n");
	fileStream << ("mount -o rw,p6 \"$GAME\"/base/ /\n\n");

	if (SendMessage(GetDlgItem(hwndDlg, CHECKBOX_MAIN_AUTOLOAD), BM_GETCHECK, 0, 0))
	{
		fileStream << ("# Mount autoload game data\n");

		AddAutoloadGameData();

		fileStream << ("\n# Mount autoload game folder\n");
		fileStream << ("mount -o rw,p8 autoload/ /\n");
		fileStream << ("mount -o rw,p8 autoload/base/ /\n\n");
	}

	fileStream << ("# Execute initialization scripts\n");
	fileStream << ("exec input.cfg\nexec config.cfg\n");

	fileStream.close();

	// Save Launcher Settings
	SaveLauncherSettings(hwndDlg);

	// Run Game
	RunProgram(NULL, const_cast<char *>(cmdLine.c_str()), false);
	EndDialog(hwndDlg, 0);
	exit(0);
}

HWND CreateToolTip(int toolID, HWND hDlg, PTSTR pszText)
{
    if (!toolID || !hDlg || !pszText)
    {
        return FALSE;
    }
    // Get the window of the tool.
    HWND hwndTool = GetDlgItem(hDlg, toolID);
    
    // Create the tooltip. g_hInst is the global instance handle.
    HWND hwndTip = CreateWindowEx(NULL, TOOLTIPS_CLASS, NULL,
                              WS_POPUP | TTS_ALWAYSTIP,
                              CW_USEDEFAULT, CW_USEDEFAULT,
                              CW_USEDEFAULT, CW_USEDEFAULT,
                              hDlg, NULL, 
                              hInst, NULL);
    
   if (!hwndTool || !hwndTip)
   {
       return (HWND)NULL;
   }                              
                              
    // Associate the tooltip with the tool.
    TOOLINFO toolInfo = { 0 };
    toolInfo.cbSize = sizeof(toolInfo);
    toolInfo.hwnd = hDlg;
    toolInfo.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
    toolInfo.uId = (UINT_PTR)hwndTool;
    toolInfo.lpszText = pszText;
    SendMessage(hwndTip, TTM_ADDTOOL, 0, (LPARAM)&toolInfo);

    return hwndTip;
}

BOOL CALLBACK DialogProc_Advanced (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		// Fullscreen Refresh Rate
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_ADV_FSRR), CB_ADDSTRING, 0, (LPARAM) _T("Most Compatible"));
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_ADV_FSRR), CB_ADDSTRING, 0, (LPARAM) _T("75 Hertz"));
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_ADV_FSRR), CB_ADDSTRING, 0, (LPARAM) _T("85 Hertz"));
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_ADV_FSRR), CB_ADDSTRING, 0, (LPARAM) _T("Fastest Supported"));
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_ADV_FSRR), CB_SELECTSTRING, -1, (LPARAM)ReadIni("Advanced", "FullscreenRefreshRate", "Most Compatible", "./launcher.ini").c_str());

		// Load Settings
		if (!_stricmp(ReadIni("Advanced", "TextureDetail", "High", "./launcher.ini").c_str(), "Medium"))
			SendMessage(GetDlgItem(hwndDlg, RADIO_ADV_MED), BM_SETCHECK, BST_CHECKED, 0);
		else if (!_stricmp(ReadIni("Advanced", "TextureDetail", "High", "./launcher.ini").c_str(), "Low"))
			SendMessage(GetDlgItem(hwndDlg, RADIO_ADV_LOW), BM_SETCHECK, BST_CHECKED, 0);
		else
			SendMessage(GetDlgItem(hwndDlg, RADIO_ADV_HIGH), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Advanced", "DisablePixelFog", "0", "./launcher.ini").c_str(), "1"))
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOPIXELFOG), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Advanced", "DisableHardwareTnL", "0", "./launcher.ini").c_str(), "1"))
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWARETNL), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Advanced", "DisableHardware3DSound", "0", "./launcher.ini").c_str(), "1"))
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWARE3DSND), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Advanced", "DisableHardwareTextureTransforms", "0", "./launcher.ini").c_str(), "1"))
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWARETEXTRANSFORMS), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Advanced", "DisableHardwareEnvironmentMapping", "0", "./launcher.ini").c_str(), "1"))
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOHARDWAREENVMAPPING), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Advanced", "DisableCompiledVertexArrays", "0", "./launcher.ini").c_str(), "1"))
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_ADV_NOCOMPILEDVERTARRAYS), BM_SETCHECK, BST_CHECKED, 0);

		CenterWindow(hwndDlg);
		return TRUE;

	case WM_CLOSE:
		SaveAdvancedLauncherSettings(hwndDlg);
		advancedMenuOpen = false;
		EnableWindow(GetWindow(hwndDlg, GW_OWNER), true);
		EndDialog(hwndDlg, 0);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case BUTTON_ADV_OK:
			SaveAdvancedLauncherSettings(hwndDlg);
			advancedMenuOpen = false;
			EnableWindow(GetWindow(hwndDlg, GW_OWNER), true);
			EndDialog(hwndDlg, 0);
			return TRUE;

		default:
			break;
		}
	}
	return FALSE;
}

BOOL CALLBACK DialogProc_Main (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		// Tooltips
		CreateToolTip(RADIO_MAIN_FULLSCREEN, hwndDlg, "The game is usually faster in fullscreen");
		CreateToolTip(RADIO_MAIN_WINDOWED, hwndDlg, "The game is usually faster in fullscreen");

		CreateToolTip(RADIO_MAIN_16BIT, hwndDlg, "32-bit color depth offers better visual quality but is slower than 16-bit");
		CreateToolTip(RADIO_MAIN_32BIT, hwndDlg, "32-bit color depth offers better visual quality but is slower than 16-bit");

		// Renderer
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_RENDERER), CB_ADDSTRING, 0, (LPARAM) _T("OpenGL"));
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_RENDERER), CB_ADDSTRING, 0, (LPARAM) _T("DirectX 8"));
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_RENDERER), CB_ADDSTRING, 0, (LPARAM) _T("DirectX 9"));
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_RENDERER), CB_SELECTSTRING, -1, (LPARAM)ReadIni("Launcher", "Renderer", "OpenGL", "./launcher.ini").c_str());

		// Resolution
		// Populate Resolution List
		ZeroMemory(&dm, sizeof(dm));
		dm.dmSize = sizeof(dm);
		for (int iModeNum = 15; EnumDisplaySettings(NULL, iModeNum, &dm) != 0; iModeNum++)
		{
			if (prevPelsWidth != dm.dmPelsWidth || prevPelsHeight != dm.dmPelsHeight)
			{
				resolutionString = IntToStr(dm.dmPelsWidth) + "x" + IntToStr(dm.dmPelsHeight);
				SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_RESOLUTION), CB_ADDSTRING, 0, (LPARAM) _T(resolutionString.c_str()));
			}
			prevPelsWidth = dm.dmPelsWidth;
			prevPelsHeight = dm.dmPelsHeight;
		}
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_RESOLUTION), CB_SELECTSTRING, -1, (LPARAM)ReadIni("Launcher", "Resolution", "640x480", "./launcher.ini").c_str());

		// Game Directory
		PopulateGameDirList(hwndDlg);
		SendMessage(GetDlgItem(hwndDlg, COMBOBOX_MAIN_GAMEDIR), CB_SELECTSTRING, -1, (LPARAM)ReadIni("Launcher", "GameDir", "duke", "./launcher.ini").c_str());
		
		// Load Settings
		if (!_stricmp(ReadIni("Launcher", "ColorDepth", "32", "./launcher.ini").c_str(), "16"))
			SendMessage(GetDlgItem(hwndDlg, RADIO_MAIN_16BIT), BM_SETCHECK, BST_CHECKED, 0);
		else
			SendMessage(GetDlgItem(hwndDlg, RADIO_MAIN_32BIT), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Launcher", "Fullscreen", "1", "./launcher.ini").c_str(), "0"))
			SendMessage(GetDlgItem(hwndDlg, RADIO_MAIN_WINDOWED), BM_SETCHECK, BST_CHECKED, 0);
		else
			SendMessage(GetDlgItem(hwndDlg, RADIO_MAIN_FULLSCREEN), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Launcher", "UseAutoload", "0", "./launcher.ini").c_str(), "1"))
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_MAIN_AUTOLOAD), BM_SETCHECK, BST_CHECKED, 0);

		if (!_stricmp(ReadIni("Launcher", "SkipDialog", "0", "./launcher.ini").c_str(), "1"))
			PlayGame(hwndDlg);
		else
			SendMessage(GetDlgItem(hwndDlg, CHECKBOX_MAIN_ALWAYSSHOW), BM_SETCHECK, BST_CHECKED, 0);

		CenterWindow(hwndDlg);
		return TRUE;

	case WM_CLOSE:
		EndDialog(hwndDlg, 0);
		exit(0);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case BUTTON_MAIN_PLAY:
			PlayGame(hwndDlg);
			return TRUE;

		case BUTTON_MAIN_EDITOR:
			RunProgram("./prismed.exe", NULL, false);
			EndDialog(hwndDlg, 0);
			exit(0);
			return TRUE;

		case BUTTON_MAIN_ADVANCED:
			if (!advancedMenuOpen)
			{
				advancedMenuOpen = true;
				EnableWindow(hwndDlg, false);
				DialogBox(hInst, MAKEINTRESOURCE(135), hwndDlg, (DLGPROC)DialogProc_Advanced);
			}
			return TRUE;

		case BUTTON_MAIN_EXIT:
			SaveLauncherSettings(hwndDlg);
			EndDialog(hwndDlg, 0);
			exit(0);
			return TRUE;

		default:
			return FALSE;
		}
	}
	return FALSE;
}

int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	advancedMenuOpen = false;
    hInst = hInstance;
    return DialogBox(hInstance, MAKEINTRESOURCE(102), NULL, (DLGPROC)DialogProc_Main);
}